@@ -139,7 +139,12 @@ public class ControllerListener extends TCPListener {
 			sin=socket.getInputStream();
 			_out=new PrintStream(socket.getOutputStream());
 			fireConnected();
+			if(listener!=null) {
+				listener.gotConnection();
+				//listener.repaint();
+			}
 			_out.println("!hello");
+			_out.println("!dump_stack");
 			_out.println("!refresh");
 			while (true) {
 				String msgtype=readLine(sin);
@@ -186,6 +191,19 @@ public class ControllerListener extends TCPListener {
 						_connectCount=Integer.parseInt(readLine(sin));
 						if(!_isConnected) break;
 						continue;
+					} else if(msgtype.equals("stack_dump")) {
+						_titles.clear();
+						int depth=Integer.parseInt(readLine(sin));
+						if(_menus.size()>depth) {
+							System.err.println("Bad stack_dump - more menus than titles!");
+							_isConnected=false;
+							break;
+						}
+						for(int i=_menus.size(); i<depth; i++)
+							_menus.insertElementAt(new Vector(),0);
+						for(int i=0; i<depth && _isConnected; i++)
+							_titles.add(readLine(sin));
+						if(!_isConnected) break;
 					} else if(msgtype.equals("goodbye")) {
 						//removeDynObjs();
 						System.out.println("Remote is shutting down.");
@@ -254,8 +272,8 @@ public class ControllerListener extends TCPListener {
 							System.out.println("ControllerGUI: error - could not close "+name+" - not loaded");
 						} else {
 							try { 
-								Method m=obj.getClass().getMethod("close",null);
-								m.invoke(obj,null);
+								Method m=obj.getClass().getMethod("close",(java.lang.Class[])null);
+								m.invoke(obj,(java.lang.Object[])null);
 							} catch (Exception e) {}
 						}
 						dynObjs.remove(name);
@@ -265,11 +283,13 @@ public class ControllerListener extends TCPListener {
 						System.err.println("ControllerListener - Invalid message type:"+msgtype);
 					}
 					_updatedFlag=true;
-					if(listener!=null)
-						listener.repaint();
-					else
-						System.out.println("null listener");
-				}
+				} //end of synchronized update section
+				if(listener!=null) {
+					//System.out.println("Listener repaint");
+					//listener.repaint();
+					listener.updated();
+				} else
+					System.out.println("null listener");
 			}
 		} catch (SocketException e) {
 		} catch (Exception e) {
@@ -288,7 +308,7 @@ public class ControllerListener extends TCPListener {
 		_status="Reconnecting.";
 		if(listener!=null) {
 			listener.lostConnection();
-			listener.repaint();
+			//listener.repaint();
 		}
 		//The sleep is to get around the socket still listening after being closed thing
 		//if(!destroy)
@@ -301,8 +321,8 @@ public class ControllerListener extends TCPListener {
 		while(it.hasNext()) {
 			Object obj=it.next();
 			try {
-				Method m=obj.getClass().getMethod("close",null);
-				m.invoke(obj,null);
+				Method m=obj.getClass().getMethod("close",(java.lang.Class[])null);
+				m.invoke(obj,(java.lang.Object[])null);
 			} catch(Exception ex) {}
 		}
 		dynObjs.clear();
