@@ -15,6 +15,8 @@ import javax.imageio.ImageReader;
 import javax.imageio.stream.ImageInputStream;
 import javax.imageio.stream.MemoryCacheImageInputStream;
 
+import java.awt.Color;
+
 public class TCPVisionListener extends TCPListener implements VisionListener {
 	boolean updatedFlag;
 	Date timestamp;
@@ -23,7 +25,7 @@ public class TCPVisionListener extends TCPListener implements VisionListener {
 	protected Vector listeners = new Vector();
 	protected boolean updating;
 
-	public void addListener(VisionUpdatedListener l) { listeners.add(l); }
+	public void addListener(VisionUpdatedListener l) { listeners.add(l); needConnection(); }
 	public void removeListener(VisionUpdatedListener l) { listeners.remove(l); }
 	public void fireVisionUpdate() {
 		updating=true;
@@ -71,6 +73,19 @@ public class TCPVisionListener extends TCPListener implements VisionListener {
 
 	Object packetFormatChangeLock=new Object();
 
+
+    boolean readingImage;
+
+    public void setReadingImage()
+    {
+	readingImage = true;
+    }
+
+    public boolean isReadingImage()
+    {
+	return readingImage;
+    }
+
 	public boolean hasRawJPEG() { return isJPEG; }
 	public byte[] getJPEG() { return _jpeg; }
 	public int getJPEGLen() { return jpegLen; }
@@ -340,11 +355,69 @@ public class TCPVisionListener extends TCPListener implements VisionListener {
 		isIndex=true;
 		return true;
 	}
+    
+    public boolean readRegions(InputStream in, int chanW, int chanH) throws java.io.IOException {
+        //clear the _data array
+        for(int h=0; h<chanH; h++) {
+            for(int w =0; w<chanW; w++) {
+                _data[chanW*h+w]=0;
+            }
+        }
+
+        int numColors=readInt(in);
+        if(!_isConnected) return false;
+        for(int curColor = 0; curColor < numColors; curColor++) {
+            int numRegions = readInt(in);
+            if(!_isConnected) return false;
+            
+        	readBytes(_tmp,in,36*numRegions);
+			if(!_isConnected) return false;
+            	
+			int dpos=0;
+			for (int i = 0; i<numRegions; i++) {
+				byte color= _tmp[dpos];
+	            dpos +=4;
+	            int x1 = byteToInt(_tmp,dpos);
+	            dpos +=4;
+	            int y1 = byteToInt(_tmp,dpos);
+	            dpos +=4;
+	            int x2 = byteToInt(_tmp,dpos);
+	            dpos +=4;
+	            int y2 = byteToInt(_tmp,dpos);
+                //The  data of the centroid, area and run_start are ignored
+	            dpos +=20; //isn't nessescary, but now it nicely adds up to 36
+                if (  x1 > chanW || y1 > chanH || x2 > chanW || y2 > chanH
+                   || x1 > x2 || y1 > y2
+                   || x1 < 0 || x2 < 0 || y1 < 0 || y2 < 0
+                   )
+					return false;
+				
+                //Fill the data array with the bounding boxes..
+                //..the top and bottom lines
+				for (int tb = x1; tb <= x2; tb++) {
+	                _data[y1*width+tb]=color;
+	                _data[y2*width+tb]=color;
+	            }
+	            //..the left and right lines
+				for (int lr = y1; lr <= y2; lr++) {
+	                _data[lr*width+x1]=color;
+	                _data[lr*width+x2]=color;
+	            }
+			}
+            readBytes(_tmp,in,12); //read out the min_area, total_area and merge_threshhold and ignore them:)
+        }
+        if(!readColorModel(in)) return false;
+		if(!_isConnected) return false;
+		isIndex=true;
+		return true;
+	}
 
 	public void connected(Socket socket) {
 		_isConnected=true;
-		while(listeners==null) //fixes a race condition, since Listener constructor spawns thread, subclass constructors may not be finished
+		while(listeners==null) {
+			System.out.println("Assert: Bad race condition -- shouldn't be happening");
 			Thread.yield();
+		}
 		fireVisionUpdate();
 		try {
 			InputStream in=socket.getInputStream();
@@ -353,7 +426,8 @@ public class TCPVisionListener extends TCPListener implements VisionListener {
 				String type = readLoadSaveString(in);
 				if(!_isConnected) break; //System.out.println("Got type="+type);
 				if(!type.equals("TekkotsuImage")) {
-					System.err.println("Unrecognized type: "+type);
+					if(!type.equals("CloseConnection"))
+						System.err.println("Unrecognized type: "+type);
 					break;
 				}
 				fireConnected();
@@ -457,17 +531,27 @@ public class TCPVisionListener extends TCPListener implements VisionListener {
 						} else if(fmt.equals("SegColorImage")) {
 							isJPEG=false;
 							isIndex=true;
-							if(!readIndexedColor(in,chanwidth,chanheight)) { failed=true; System.err.println("TCPVisionListener SegColor read failed"); break; }
+							if(!readIndexedColor(in,chanwidth,chanheight)) { 
+							    failed=true; 
+							    System.err.println("TCPVisionListener SegColor read failed"); 
+							    break; 
+							}
 						} else if(fmt.equals("RLEImage")) {
 							isJPEG=false;
 							isIndex=true;
 							if(!readRLE(in,chanwidth,chanheight)) { failed=true; System.err.println("TCPVisionListener RLEImage read failed"); break; }
+						} else if(fmt.equals("RegionImage")) {
+							isJPEG=false;
+							isIndex=true;
+							if(!readRegions(in,chanwidth,chanheight)) { failed=true; System.err.println("TCPVisionListener RegionImage read failed"); break; }
 						} else {
 							isJPEG=false;
 							isIndex=false;
 							System.err.println("Unrecognized format: "+fmt);
 							failed=true; break;
 						}
+						//System.out.println("Done reading");
+						readingImage = false;
 					}
 				}
 				if(failed || !_isConnected) {
@@ -503,8 +587,8 @@ public class TCPVisionListener extends TCPListener implements VisionListener {
 	}
 	
 	public byte[] getData() {
-//		frameTimer();
-		synchronized (_outd) {
+//		frameTimer();		
+	    synchronized (_outd) {
 			updatedFlag=false;
 			return _outd;
 		}
@@ -621,3 +705,4 @@ public class TCPVisionListener extends TCPListener implements VisionListener {
 	public TCPVisionListener(int port) { super(port); }
 	public TCPVisionListener(String host, int port) { super(host,port); }
 }
+
