@@ -68,7 +68,7 @@ public class WorldStateJointsListener extends TCPListener {
     return _isConnected;
   }
 
-  public WorldStateJointsListener() { super(); }
-  public WorldStateJointsListener(int port) { super(port); }
-  public WorldStateJointsListener(String host, int port) { super(host,port); }
+  public WorldStateJointsListener() { super(); needConnection(); }
+  public WorldStateJointsListener(int port) { super(port); needConnection(); }
+  public WorldStateJointsListener(String host, int port) { super(host,port); needConnection(); }
 }
