@@ -55,7 +55,7 @@ public class WorldStatePIDsListener extends TCPListener {
     return _isConnected;
   }
 
-  public WorldStatePIDsListener() { super(); }
-  public WorldStatePIDsListener(int port) { super(port); }
-  public WorldStatePIDsListener(String host, int port) { super(host,port); }
+  public WorldStatePIDsListener() { super(); needConnection(); }
+  public WorldStatePIDsListener(int port) { super(port); needConnection(); }
+  public WorldStatePIDsListener(String host, int port) { super(host,port); needConnection(); }
 }
